@@ -21,7 +21,7 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: ansi.c,v 1.12 1993/07/21 15:42:44 mlschroe Exp $ FAU")
+RCS_ID("$Id: ansi.c,v 1.16 1993/08/05 14:23:28 mlschroe Exp $ FAU")
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -44,7 +44,7 @@ extern struct display *display, *displays;
 
 extern int  force_vt;
 extern int  all_norefresh;	/* => display */
-extern char Esc, MetaEsc;	/* => display */
+extern int  ZombieKey;
 extern time_t Now;
 
 int maxwidth;			/* maximum of all widths so far */
@@ -142,8 +142,25 @@ int *lenp;
   char *ibuf;
   
   fore = d_fore;
-#ifdef MULTIUSER
   /* if w_wlock is set, only one user may write, else we check acls */
+  if (fore->w_ptyfd < 0)
+    {
+      SetCurr(fore);
+      Special('\007');
+      while ((*lenp)-- > 0)
+        {
+	  if (*(*bufpp)++ == ZombieKey)
+	    {
+	      debug2("Turning undead: %d(%s)\n", fore->w_number, fore->w_title);
+	      KillWindow(fore);
+	      break;
+	    }
+	}
+      *bufpp += *lenp;
+      *lenp = 0;
+      return;
+    }
+#ifdef MULTIUSER
   if ((fore->w_wlock == WLOCK_OFF) ? 
       AclCheckPermWin(d_user, ACL_WRITE, fore) :
       (d_user != fore->w_wlockuser))
@@ -396,7 +413,7 @@ register int len;
       /* The next part is only for speedup */
       if (curr->w_state == LIT &&
           c >= ' ' && ((c & 0x80) == 0 || display == 0 || !CB8) &&
-          !curr->w_insert && !curr->w_ss)
+          !curr->w_insert && !curr->w_ss && curr->w_x < cols - 1)
 	{
 	  register int currx;
 	  register char *imp, *atp, *fop, at, fo;
@@ -540,7 +557,7 @@ skip:	      if (--len == 0)
 		    AddStr(curr->w_string);
 		  break;
 		case AKA:
-		  if (curr->w_aka == curr->w_akabuf && !*curr->w_string)
+		  if (curr->w_title == curr->w_akabuf && !*curr->w_string)
 		    break;
 		  ChangeAKA(curr, curr->w_string, 20);
 		  if (!*curr->w_string)
@@ -730,7 +747,16 @@ skip:	      if (--len == 0)
 		}
 	      else
 		{
-		  LineFeed(display == 0 || AM ? 0 : 2);
+		  if (display && d_x != cols)	/* write char again */
+		    {
+		      SetAttrFont(curr->w_attr[curr->w_y][cols - 1],
+                                  curr->w_font[curr->w_y][cols - 1]);
+		      RAW_PUTCHAR(curr->w_image[curr->w_y][cols - 1]);
+		      SetAttrFont(curr->w_Attr, curr->w_Font);
+		      if (curr->w_y == d_bot)
+			d_lp_missing = 0;	/* just wrote it */
+		    }
+		  LineFeed((display == 0 || AM) ? 0 : 2);
 		  if (display)
 		    PUTCHAR(c);
 		  SetChar(c);
@@ -1064,7 +1090,9 @@ int c, intermediate;
 	    }
 	  break;
 	case 'n':
-	  if (a1 == 6)		/* Report cursor position */
+	  if (a1 == 5)		/* Report terminal status */
+	    Report("\033[0n", 0, 0);
+	  else if (a1 == 6)		/* Report cursor position */
 	    Report("\033[%d;%dR", curr->w_y + 1, curr->w_x + 1);
 	  break;
 	case 'c':		/* Identify as VT100 */
@@ -1073,56 +1101,60 @@ int c, intermediate;
 	}
       break;
     case '?':
-      debug2("\\E[?%d%c\n",a1,c);
-      if (c != 'h' && c != 'l')
-	break;
-      i = (c == 'h');
-      switch (a1)
+      for (a2 = 0; a2 < curr->w_NumArgs; a2++)
 	{
-	case 1:
-	  CursorkeysMode(curr->w_cursorkeys = i);
-#ifndef TIOCPKT
-	  NewAutoFlow(curr, !i);
-#endif /* !TIOCPKT */
-	  break;
-	case 3:
-	  i = (i ? Z0width : Z1width);
-	  if (curr->w_width != i && (display == 0 || (CZ0 || CWS)))
+	  a1 = curr->w_args[a2];
+	  debug2("\\E[?%d%c\n",a1,c);
+	  if (c != 'h' && c != 'l')
+	    break;
+	  i = (c == 'h');
+	  switch (a1)
 	    {
-              ChangeWindowSize(curr, i, curr->w_height);
-	      SetCurr(curr);	/* update rows/cols */
+	    case 1:
+	      CursorkeysMode(curr->w_cursorkeys = i);
+    #ifndef TIOCPKT
+	      NewAutoFlow(curr, !i);
+    #endif /* !TIOCPKT */
+	      break;
+	    case 3:
+	      i = (i ? Z0width : Z1width);
+	      if (curr->w_width != i && (display == 0 || (CZ0 || CWS)))
+		{
+		  ChangeWindowSize(curr, i, curr->w_height);
+		  SetCurr(curr);	/* update rows/cols */
+		  if (display)
+		    Activate(0);
+		}
+	      break;
+	    case 5:
+	      if (i)
+		curr->w_vbwait = 1;
+	      else
+		{
+		  if (display && curr->w_vbwait)
+		    PutStr(VB);
+		  curr->w_vbwait = 0;
+		}
+	      break;
+	    case 6:
+	      if ((curr->w_origin = i) != 0)
+		{
+		  curr->w_y = curr->w_top;
+		  curr->w_x = 0;
+		}
+	      else
+		curr->w_y = curr->w_x = 0;
 	      if (display)
-		Activate(0);
+		GotoPos(curr->w_x, curr->w_y);
+	      break;
+	    case 7:
+	      curr->w_wrap = i;
+	      break;
+	    case 35:
+	      debug1("Cursor %svisible\n", i ? "in" : "");
+	      curr->w_cursor_invisible = i;
+	      break;
 	    }
-	  break;
-	case 5:
-	  if (i)
-	    curr->w_vbwait = 1;
-	  else
-	    {
-	      if (display && curr->w_vbwait)
-		PutStr(VB);
-	      curr->w_vbwait = 0;
-	    }
-	  break;
-	case 6:
-	  if ((curr->w_origin = i) != 0)
-	    {
-	      curr->w_y = curr->w_top;
-	      curr->w_x = 0;
-	    }
-	  else
-	    curr->w_y = curr->w_x = 0;
-	  if (display)
-	    GotoPos(curr->w_x, curr->w_y);
-	  break;
-	case 7:
-	  curr->w_wrap = i;
-	  break;
-	case 35:
-	  debug1("Cursor %svisible\n", i ? "in" : "");
-	  curr->w_cursor_invisible = i;
-	  break;
 	}
       break;
     }
@@ -1874,10 +1906,10 @@ int l;
     l = 20;
   strncpy(p->w_akachange, s, l);
   p->w_akachange[l] = 0;
-  p->w_aka = p->w_akachange;
+  p->w_title = p->w_akachange;
   if (p->w_akachange != p->w_akabuf)
     if (p->w_akachange[0] == 0 || p->w_akachange[-1] == ':')
-      p->w_aka = p->w_akabuf + strlen(p->w_akabuf) + 1;
+      p->w_title = p->w_akabuf + strlen(p->w_akabuf) + 1;
 }
 
 static void
