@@ -21,7 +21,7 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: attacher.c,v 1.3 1993/07/21 15:42:48 mlschroe Exp $ FAU")
+RCS_ID("$Id: attacher.c,v 1.5 1993/08/03 22:22:37 jnweiger Exp $ FAU")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -549,8 +549,19 @@ Attacher()
 #endif
   for (;;)
     {
+#ifdef DEBUG
+      sleep(30);
+      if (kill(MasterPid, 0) < 0 && errno != EPERM)
+        {
+	  debug1("attacher: Panic! MasterPid %d does not exist.\n", MasterPid);
+	  AttacherPanic++;
+	}
+#else
       pause();
-      debug("attacher: huh! a signal!\n");
+#endif
+/*
+      debug("attacher: ding!\n");
+*/
 #ifdef DEBUG
       if (AttacherPanic)
         {
