@@ -18,138 +18,9 @@
  * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  ****************************************************************
- * $Id: extern.h,v 1.10 1993/07/21 15:43:08 mlschroe Exp $ FAU
+ * $Id: extern.h,v 1.13 1993/08/04 00:42:34 mlschroe Exp $ FAU
  */
 
-/****************************************************************
- * Thanks to Christos S. Zoulas (christos@ee.cornell.edu) who 
- * mangled the screen source through 'gcc -Wall'.
- ****************************************************************
- */
-
-#ifndef MEMFUNCS_DECLARED /* bsd386 */
-# ifndef SYSV
-extern void  bzero __P((char *, int));
-extern void  bcopy __P((char *, char *, int));
-# else
-extern char *memset __P((char *, int, size_t));
-extern char *memcpy __P((char *, char *, int));
-#  ifdef USEBCOPY
-extern void  bcopy __P((char *, char *, int));
-#  endif
-# endif
-#endif /* MEMFUNCS_DECLARED */
-#ifndef WAITSTUFF_DECLARED
-# ifdef BSDWAIT
-struct rusage;
-union wait;
-extern int   wait3 __P((union wait *, int, struct rusage *));
-# else /* BSDWAIT */
-extern pid_t waitpid __P((int, int *, int));
-# endif /* BSDWAIT */
-#endif /* WAITSTUFF_DECLARED */
-#ifndef GETDTABLESIZE_DECLARED
-extern int   getdtablesize __P((void));
-#endif /* GETDTABLESIZE_DECLARED */
-#ifndef REUID_DECLARED
-# if !defined(NOREUID)
-#  ifdef hpux
-extern int   setresuid __P((uid_t, uid_t, uid_t));
-extern int   setresgid __P((gid_t, gid_t, gid_t));
-#  else
-extern int   setreuid __P((uid_t, uid_t));
-extern int   setregid __P((gid_t, gid_t));
-#  endif
-# endif
-#endif /* REUID_DECLARED */
-#ifndef CRYPT_DECLARED
-extern char *crypt __P((char *, char *));
-#endif /* CRYPT_DECLARED */
-#ifdef sun
-# ifdef SVR4 /* Solaris 2 */
-extern pid_t   getpgrp __P((int));
-# else
-extern int   getpgrp __P((int));
-# endif
-#endif
-#ifndef MKNOD_DECLARED
-# ifdef POSIX
-extern int   mknod __P((const char *, mode_t, dev_t));
-# else
-extern int   mknod __P((char *, int, int));
-# endif
-#endif /* MKNOD_DECLARED */
-#ifndef PUTENV_DECLARED
-extern int   putenv __P((char *));
-#endif /* PUTENV_DECLARED */
-#ifndef KILLSTUFF_DECLARED
-extern int   kill __P((int, int));
-# ifndef SYSV
-extern int   killpg __P((int, int));
-# endif
-#endif /* KILLSTUFF_DECLARED */
-extern int   tgetent __P((char *, char *));
-extern int   tgetnum __P((char *));
-extern int   tgetflag __P((char *));
-extern void  tputs __P((char *, int, void (*)(int)));
-# ifdef POSIX
-extern pid_t setsid __P((void));
-#  ifndef SETPGID_DECLARED
-extern int   setpgid __P((int, int));
-#  endif /* SETPGID_DECLARED */
-#  if defined(BSDI) || defined(__386BSD__) || defined(_CX_UX) || defined(SVR4)
-extern int   tcsetpgrp __P((int, pid_t));
-#  else
-extern int   tcsetpgrp __P((int, int));
-#  endif
-# endif /* POSIX */
-extern pid_t getpid __P((void));
-extern uid_t getuid __P((void)); 
-extern uid_t geteuid __P((void));
-extern gid_t getgid __P((void)); 
-extern gid_t getegid __P((void));
-extern int   isatty __P((int)); 
-#ifdef notdef
-extern int   chown __P((const char *, uid_t, gid_t)); 
-#endif
-#ifndef GETHOSTNAME_DECLARED
-extern int   gethostname __P((char *, size_t));
-#endif /* GETHOSTNAME_DECLARED */
-extern off_t lseek __P((int, off_t, int));
-#if defined(sun) && !defined(__GNUC__)		/* sun's exit returns ??? */
-extern int   exit __P((int));
-#else
-extern void  exit __P((int));
-#endif
-extern char *getwd __P((char *));
-extern char *getenv __P((const char *));
-extern time_t time __P((time_t *));
-
-#if defined(linux) || defined(SVR4)
-extern char *getpass __P((const char *));
-#else
-extern char *getpass __P((char *));
-#endif
-extern char *getlogin(), *ttyname();
-extern int   fflush(); 
-#if !defined(__STDC__) || !defined(POSIX)
-extern char *malloc(), *realloc();
-#endif
-
-#ifndef INDEX_DECLARED
-# if defined(SVR4) && defined(sun)
-extern char *index __P((const char *, int));
-extern char *rindex __P((const char *, int));
-# else
-extern char *index __P((char *, int));
-extern char *rindex __P((char *, int));
-# endif
-#endif /* INDEX_DECLARED */
-
-
-/************************************************************
- *  Screens function declarations
- */
 
 /* screen.c */
 extern void  main __P((int, char **));
@@ -232,6 +103,7 @@ extern void  Input __P((char *, int, void (*)(), int));
 extern void  exit_with_usage __P((char *));
 extern void  display_help __P((void));
 extern void  display_copyright __P((void));
+extern void  display_displays __P((void));
 
 /* window.c */
 extern int   MakeWindow __P((struct NewWindow *));
@@ -240,6 +112,9 @@ extern void  FreeWindow __P((struct win *));
 extern int   winexec __P((char **));
 extern void  FreePseudowin __P((struct win *));
 #endif
+#ifdef MULTI
+extern int   execclone __P((char **));
+#endif
 extern void  nwin_compose __P((struct NewWindow *, struct NewWindow *, struct NewWindow *));
 
 /* utmp.c */
@@ -268,11 +143,12 @@ extern int   OpenPTY __P((char **));
 extern void  InitKeytab __P((void));
 extern void  ProcessInput __P((char *, int));
 extern int   FindCommnr __P((char *));
-extern void  DoAction __P((struct action *, int));
+extern void  DoCommand __P((char **));
 extern void  KillWindow __P((struct win *));
+extern int   ReleaseAutoWritelock __P((struct display *, struct win *));
 extern void  SetForeWindow __P((struct win *));
 extern int   Parse __P((char *, char **));
-extern int   ParseEscape __P((char *));
+extern int   ParseEscape __P((struct user *, char *));
 extern void  DoScreen __P((char *, char **));
 extern void  ShowWindows __P((void));
 extern int   WindowByNoN __P((char *));
@@ -383,6 +259,7 @@ extern int   UserStatus __P((void));
 extern int   AclInit __P((char *));
 extern int   AclSetPass __P((char *, char *));
 extern int   AclDelUser __P((char *));
+extern int   UserFreeCopyBuffer __P((struct user *));
 extern int   AclAddGroup __P((char *));
 extern int   AclSetGroupPerm __P((char *, char *));
 extern int   AclDelGroup __P((char *));
