@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  ****************************************************************
- * $Id: patchlevel.h,v 1.4 1993/07/21 15:43:16 mlschroe Exp $ FAU
+ * $Id: patchlevel.h,v 1.8 1993/08/05 14:23:58 mlschroe Exp $ FAU
  */
 
 /****************************************************************
@@ -121,11 +121,17 @@
  *                     now works without nethackoption set.
  * 01.07.93 -- 3.04.02 Implementing real acls.
  * 22.07.93 -- 3.05.00 Fixed SVR4, some multiuser bugs, -- DISTRIBUTED
+ * 08.08.93 -- 3.05.01 ${srcdir} feature added. Shellprog bug fixed.
+ *                     Motorola reattach bug fixed. Writelock bug fixed.
+ *                     Copybuffer moved into struct user. Configure.in
+ *                     uglified for Autoconf1.5. Paste may now have an
+ *                     argument. Interactive setenv. Right margin bug
+ *                     fixed. IRIX 5 patches. -- DISTRIBUTED
  */
 
 #define ORIGIN "FAU"
 #define REV 3
 #define VERS 5
-#define PATCHLEVEL 0
-#define DATE "22-Jul-93"
+#define PATCHLEVEL 1
+#define DATE "8-Aug-93"
 #define STATE "" 
