@@ -21,7 +21,7 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: process.c,v 1.12 1993/07/21 15:43:19 mlschroe Exp $ FAU")
+RCS_ID("$Id: process.c,v 1.18 1993/08/05 14:24:01 mlschroe Exp $ FAU")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -51,7 +51,6 @@ extern char *BellString, *ActivityString, *ShellProg, *ShellArgs[];
 extern char *hardcopydir, *screenlogdir;
 extern char *VisualBellString;
 extern int VBellWait, MsgWait, MsgMinWait, SilenceWait;
-extern char Esc, MetaEsc;
 extern char SockPath[], *SockNamePtr;
 extern int TtyMode, auto_detach;
 extern int iflag;
@@ -59,10 +58,10 @@ extern int default_wrap;
 extern int use_hardstatus, visual_bell, default_monitor;
 extern int default_startup;
 extern int slowpaste, defobuflimit;
+extern int ZombieKey;
 #ifdef AUTO_NUKE
 extern int defautonuke;
 #endif
-extern char screenterm[];
 extern int intrc, origintrc; /* display? */
 extern struct NewWindow nwin_default, nwin_undef;
 #ifdef COPY_PASTE
@@ -75,15 +74,6 @@ extern char *BufferFile, *PowDetachString;
 #endif
 extern time_t Now;
 
-/*
- * XXX: system
- */
-extern time_t time __P((time_t *));
-#if !defined(BSDI) && !defined(SVR4) && !defined(linux) && !defined(__386BSD__)
-extern char *getpass __P((char *));
-#endif /* !BSDI && !SVR4 && !linux && !__386BSD__*/
-
-
 static int  CheckArgNum __P((int, char **));
 static void FreeKey __P((int));
 static int  NextWindow __P((void));
@@ -95,6 +85,8 @@ static void ShowInfo __P((void));
 static void SwitchWindow __P((int));
 static char **SaveArgs __P((char **));
 static struct win *WindowByName __P((char *));
+static int  WindowByNumber __P((char *));
+static void DoAction  __P((struct action *, int));
 static int  ParseSwitch __P((struct action *, int *));
 static int  ParseOnOff __P((struct action *, int *));
 static int  ParseSaveStr __P((struct action *act, char **));
@@ -107,6 +99,7 @@ static int  IsNumColon __P((char *, int, char *, int));
 static void InputColon __P((void));
 static void Colonfin __P((char *, int));
 static void InputSelect __P((void));
+static void InputSetenv __P((char *));
 static void InputAKA __P((void));
 static void AKAfin __P((char *, int));
 #ifdef COPY_PASTE
@@ -126,6 +119,7 @@ static void pow_detach_fn __P((char *, int));
 
 extern struct display *display, *displays;
 extern struct win *fore, *console_window, *windows;
+extern struct user *users;
 
 extern char screenterm[], HostName[], version[];
 extern struct NewWindow nwin_undef, nwin_default;
@@ -134,7 +128,6 @@ extern struct layer BlankLayer;
 
 extern int Z0width, Z1width;
 extern int real_uid, real_gid;
-extern int visual_bell, default_monitor;
 
 #ifdef NETHACK
 extern int nethackflag;
@@ -153,17 +146,8 @@ int CheckPassword;
 char Password[20];
 #endif
 
-#define MAX_PLOP_DEFS 256
+struct plop plop_tab[MAX_PLOP_DEFS];
 
-static struct plop
-{
-  char *buf;
-  int len;
-} plop_tab[MAX_PLOP_DEFS];
-
-
-char Esc = Ctrl('a');
-char MetaEsc = 'a';
 #ifdef PTYMODE
 int TtyMode = PTYMODE;
 #else
@@ -186,7 +170,7 @@ InitKeytab()
       ktab[i].args = noargs;
     }
 
-  ktab['h'].nr = ktab[Ctrl('h')].nr = RC_HARDCOPY;
+  ktab['h'].nr = RC_HARDCOPY;
 #ifdef BSDJOBS
   ktab['z'].nr = ktab[Ctrl('z')].nr = RC_SUSPEND;
 #endif
@@ -204,8 +188,8 @@ InitKeytab()
   ktab['t'].nr = ktab[Ctrl('t')].nr = RC_TIME;
   ktab['i'].nr = ktab[Ctrl('i')].nr = RC_INFO;
   ktab['m'].nr = ktab[Ctrl('m')].nr = RC_LASTMSG;
-  ktab['A'].nr = RC_AKA;
-#ifdef UTMPOK
+  ktab['A'].nr = RC_TITLE;
+#if defined(UTMPOK) && defined(LOGOUTOK)
   ktab['L'].nr = RC_LOGIN;
 #endif
   ktab[','].nr = RC_LICENSE;
@@ -218,8 +202,8 @@ InitKeytab()
   ktab['C'].nr = RC_CLEAR;
   ktab['Z'].nr = RC_RESET;
   ktab['H'].nr = RC_LOG;
-  ktab[(unsigned int)Esc].nr = RC_OTHER;
-  ktab[(unsigned int)MetaEsc].nr = RC_META;
+  ktab[(int)(unsigned char)DefaultEsc].nr = RC_OTHER;
+  ktab[(int)(unsigned char)DefaultMetaEsc].nr = RC_META;
   ktab['M'].nr = RC_MONITOR;
   ktab['?'].nr = RC_HELP;
   for (i = 0; i < ((MAXWIN < 10) ? MAXWIN : 10); i++)
@@ -287,7 +271,7 @@ int ilen;
       s = ibuf;
       while (ilen > 0)
 	{
-	  if (*s++ == Esc)
+	  if (*s++ == d_user->u_Esc)
 	    break;
 	  ilen--;
 	}
@@ -342,6 +326,16 @@ char **args;
 	  return -1;
 	}
     }
+  else if ((comms[nr].flags & ARGS_PLUSONE) && (comms[nr].flags & ARGS_PLUSTWO))
+    {
+      if (i != n && i != n + 1 && i != n + 2)
+        {
+	  Msg(0, "%s: %s: %s, %s or %s argument%s required", rc_name,
+	      comms[nr].name, argss[n], argss[n + 1], argss[n + 2], 
+	      n != 0 ? "s" : "");
+	  return -1;
+	}
+    }
   else if (comms[nr].flags & ARGS_PLUSONE)
     {
       if (i != n && i != n + 1)
@@ -350,6 +344,15 @@ char **args;
           return -1;
 	}
     }
+  else if (comms[nr].flags & ARGS_PLUSTWO)
+    {
+      if (i != n && i != n + 2)
+        {
+	  Msg(0, "%s: %s: %s or %s argument%s required", rc_name, 
+	      comms[nr].name, argss[n], argss[n + 2], n != 0 ? "s" : "");
+	  return -1;
+	}
+    }
   else if (i != n)
     {
       Msg(0, "%s: %s: %s argument%s required", rc_name, comms[nr].name, argss[n], n != 1 ? "s" : "");
@@ -359,7 +362,7 @@ char **args;
 }
 
 /*ARGSUSED*/
-void
+static void
 DoAction(act, key)
 struct action *act;
 int key;
@@ -509,7 +512,7 @@ int key;
 	    break;
 	  }
 #endif
-	name = SaveStr(fore->w_aka);
+	name = SaveStr(fore->w_title);
 	KillWindow(fore);
 #ifdef NETHACK
 	if (nethackflag)
@@ -540,6 +543,185 @@ int key;
         Detach(D_POWER); /* detach and kill Attacher's parent */
       break;
 #endif
+    case RC_DEBUG:
+#ifdef DEBUG
+      if (!*args)
+        {
+	  if (dfp)
+	    Msg(0, "debugging info is written to %s/", DEBUGDIR);
+	  else
+	    Msg(0, "debugging is currently off. Use 'debug on' to enable.");
+	  break;
+	}
+      if (dfp)
+        {
+	  debug("debug: closing debug file.\n");
+	  fflush(dfp);
+	  fclose(dfp);
+	  dfp = NULL;
+	}
+      if (strcmp("off", *args))
+        {
+	  char buf[255];
+
+	  sprintf(buf, "%s/SCREEN.%d", DEBUGDIR, getpid());
+	  if ((dfp = fopen(buf, "a")) == NULL)
+	    dfp = stderr;
+	  debug("debug: opening debug file.\n");
+	}
+#else
+      Msg(0, "Sorry, screen was compiled without -DDEBUG option.");
+#endif
+      break;
+    case RC_ZOMBIE:
+      if (!(s = *args))
+        {
+	  ZombieKey = 0;
+	  break;
+	}
+      if (!(s = ParseChar(s, &ch)) || *s)
+        {
+	  Msg(0, "%s:zombie: one character expected.", rc_name);
+	  break;
+	}
+      ZombieKey = ch;
+      break;
+    case RC_WALL:
+      for (n = 0, s = *args; args[n]; n++)
+        {
+	  /* glue the vector together again. Brute force method. */
+	  while (*s)
+	    s++;
+	  while (s < args[n+1])
+	    *s++ = ' ';
+	}
+#ifdef MULTIUSER
+      s = d_user->u_name;
+#else
+      s = d_usertty;
+#endif
+      display = NULL;	/* a message without display will cause a broadcast */
+      Msg(0, "%s: %s", s, *args);
+      break;
+    case RC_AT:
+#ifdef MULTIUSER
+      s = SaveStr(d_user->u_name);
+#else
+      s = SaveStr(d_usertty);
+#endif
+      n = strlen(args[0]);
+      if (n) n--;
+      /*
+       * the windows/displays loops are quite dangerous here, take extra
+       * care not to trigger landmines. Things may appear/disappear while
+       * we are walking along.
+       */
+      switch (args[0][n])
+        {
+	case '*':
+	  {
+	    struct display *nd;
+	    struct user *u;
+
+	    args[0][n] = '\0';
+	    if (!*args[0])
+	      u = d_user;
+	    else
+	      for (u = users; u; u = u->u_next)
+	        if (!strncmp(s, u->u_name, n))
+		  break;
+	    debug1("at all displays of user %s\n", u->u_name);
+	    for (display = displays; display; display = nd)
+	      {
+		nd = display->_d_next;
+		fore = d_fore;
+	        if (d_user != u)
+		  continue;
+		debug1("AT display %s\n", d_usertty);
+		DoCommand(args + 1);
+		if (display)
+		  Msg(0, "command from %s: %s %s", 
+		      s, args[1], args[2] ? args[2] : "");
+		display = NULL;
+		fore = NULL;
+	      }
+	    free(s);
+	    return;
+	  }
+	case '%':
+	  {
+	    struct display *nd;
+
+	    args[0][n] = '\0';
+	    debug1("at display matching '%s'\n", args[0]);
+	    for (display = displays; display; display = nd)
+	      {
+	        nd = display->_d_next;
+		fore = d_fore;
+	        if (strncmp(args[0], d_usertty, n) && 
+		    (strncmp("/dev/", d_usertty, 5) || 
+		     strncmp(args[0], d_usertty + 5, n)) &&
+		    (strncmp("/dev/tty", d_usertty, 8) ||
+		     strncmp(args[0], d_usertty + 8, n)))
+		  continue;
+		debug1("AT display %s\n", d_usertty);
+		DoCommand(args + 1);
+		if (display)
+		  Msg(0, "command from %s: %s %s", 
+		      s, args[1], args[2] ? args[2] : "");
+		display = NULL;
+		fore = NULL;
+	      }
+	    free(s);
+	    return;
+	  }
+	case '#':
+	  args[0][n--] = '\0';
+	  /* FALLTHROUGH */
+	default:
+	  {
+	    struct win *nw;
+
+	    n++; 
+	    if (!*args[0] || (i = WindowByNumber(args[0])) < 0)
+	      {
+	        /* try looping over titles */
+		for (fore = windows; fore; fore = nw)
+		  {
+		    nw = fore->w_next;
+		    if (strncmp(args[0], fore->w_title, n))
+		      continue;
+		    debug2("AT window %d(%s)\n", fore->w_number, fore->w_title);
+		    i++;
+		    DoCommand(args + 1);
+		    if ((display = fore->w_display))
+		      Msg(0, "command from %s: %s %s", 
+			  s, args[1], args[2] ? args[2] : "");
+		  }
+		display = NULL;
+		fore = NULL;
+		if (i < 0)
+		  Msg(0, "%s: at '%s': no such window.\n", rc_name, args[0]);
+		free(s);
+		return;
+	      }
+	    else if (i < MAXWIN && (fore = wtab[i]))
+	      {
+	        debug2("AT window %d (%s)\n", fore->w_number, fore->w_title);
+		DoCommand(args + 1);
+		if ((display = fore->w_display))
+		  Msg(0, "command from %s: %s %s", 
+		      s, args[1], args[2] ? args[2] : "");
+		display = NULL;
+		fore = NULL;
+		free(s);
+		return;
+	      }
+	  }
+	}
+      Msg(0, "%s: at [identifier][%%|*|#] command [args]", rc_name);
+      free(s);
+      break;
 #ifdef COPY_PASTE
     case RC_COPY_REG:
       if ((s = *args) == NULL)
@@ -580,8 +762,8 @@ int key;
 
 	  if (plp->buf)
 	    free(plp->buf);
-	  plp->buf = SaveStr(args[1]);
-	  plp->len = strlen(args[1]);
+	  plp->buf = SaveStr(expand_vars(args[1]));
+	  plp->len = strlen(plp->buf);
 	}
       break;
     case RC_PROCESS:
@@ -618,7 +800,7 @@ int key;
 	SwitchWindow(d_other ? d_other->w_number : NextWindow());
       break;
     case RC_META:
-      ch = Esc;
+      ch = d_user->u_Esc;
       s = &ch;
       n = 1;
       Process(&s, &n);
@@ -714,6 +896,7 @@ int key;
 	Msg(0, "Your termcap does not specify how to change the terminal's height to %d.", n);
       break;
     case RC_AKA:
+    case RC_TITLE:
       if (*args == 0)
 	InputAKA();
       else
@@ -800,7 +983,7 @@ int key;
 	  else
 #endif
 	    Msg(0, "Window %d (%s) is now being monitored for all activity.", 
-		fore->w_number, fore->w_aka);
+		fore->w_number, fore->w_title);
 	}
       else
 	{
@@ -811,9 +994,12 @@ int key;
 	  else
 #endif
 	    Msg(0, "Window %d (%s) is no longer being monitored for activity.", 
-		fore->w_number, fore->w_aka);
+		fore->w_number, fore->w_title);
 	}
       break;
+    case RC_DISPLAYS:
+      display_displays();
+      break;
     case RC_HELP:
       display_help();
       break;
@@ -835,29 +1021,74 @@ int key;
 	  Msg(0, "Must be on a window layer");
 	  break;
 	}
-      if (GetHistory())
-	if (d_copybuffer != NULL)
-	  {
-	    fore->w_pastebuffer = d_copybuffer;
-	    fore->w_pastelen = d_copylen;
-	  }
-      break;
+      if (GetHistory() == 0)
+        break;
+      if (d_user->u_copybuffer == NULL)
+        break;
+      /*FALLTHROUGH*/
     case RC_PASTE:
-      if (d_copybuffer == NULL)
-	{
+      {
+        char *ss;
+        int l = 0;
+
+        if ((s = *args) == 0)
+          s = ".";
+        for (ss = s; (ch = *ss); ss++)
+          {
+	    if (ch == '.')
+              l += d_user->u_copylen;
+	    else
+              l += plop_tab[(int)(unsigned char)ch].len;
+          }
+        if (l == 0)
+	  {
 #ifdef NETHACK
-	  if (nethackflag)
-	    Msg(0, "Nothing happens.");
-	  else
+	    if (nethackflag)
+	      Msg(0, "Nothing happens.");
+	    else
 #endif
-	  Msg(0, "empty buffer");
-	  break;
-	}
-      fore->w_pastebuffer = d_copybuffer;
-      fore->w_pastelen = d_copylen;
-      break;
+	    Msg(0, "empty buffer");
+	    break;
+	  }
+        fore->w_pasteptr = 0;
+        fore->w_pastelen = 0;
+        if (fore->w_pastebuf)
+          free(fore->w_pastebuf);
+        fore->w_pastebuf = 0;
+        if (s[1] == 0)
+          {
+            if (*s == '.')
+	      fore->w_pasteptr = d_user->u_copybuffer;
+            else
+	      fore->w_pasteptr = plop_tab[(int)(unsigned char)ch].buf;
+	    fore->w_pastelen = l;
+	    break;
+          }
+        if ((fore->w_pastebuf = (char *)malloc(l)) == 0)
+          {
+	    Msg(0, strnomem);
+	    break;
+          }
+        l = 0;
+        for (ss = s; (ch = *ss); ss++)
+          {
+	    if (ch == '.')
+	      {
+		bcopy(d_user->u_copybuffer, fore->w_pastebuf + l, d_user->u_copylen);
+                l += d_user->u_copylen;
+              }
+	    else
+	      {
+		bcopy(plop_tab[(int)(unsigned char)ch].buf, fore->w_pastebuf + l, plop_tab[(int)(unsigned char)ch].len);
+                l += plop_tab[(int)(unsigned char)ch].len;
+              }
+          }
+        fore->w_pasteptr = fore->w_pastebuf;
+        fore->w_pastelen = l;
+        break;
+      }
     case RC_WRITEBUF:
-      if (d_copybuffer == NULL)
+      if (d_user->u_copybuffer == NULL)
 	{
 #ifdef NETHACK
 	  if (nethackflag)
@@ -877,17 +1108,17 @@ int key;
       break;
 #endif				/* COPY_PASTE */
     case RC_ESCAPE:
-      FreeKey((int)(unsigned char)Esc);
-      FreeKey((int)(unsigned char)MetaEsc);
-      if (ParseEscape(*args))
+      FreeKey((int)(unsigned char)d_user->u_Esc);
+      FreeKey((int)(unsigned char)d_user->u_MetaEsc);
+      if (ParseEscape(d_user, *args))
 	{
 	  Msg(0, "%s: two characters required after escape.", rc_name);
 	  break;
 	}
-      FreeKey((int)(unsigned char)Esc);
-      FreeKey((int)(unsigned char)MetaEsc);
-      ktab[(int)(unsigned char)Esc].nr = RC_OTHER;
-      ktab[(int)(unsigned char)MetaEsc].nr = RC_META;
+      FreeKey((int)(unsigned char)d_user->u_Esc);
+      FreeKey((int)(unsigned char)d_user->u_MetaEsc);
+      ktab[(int)(unsigned char)d_user->u_Esc].nr = RC_OTHER;
+      ktab[(int)(unsigned char)d_user->u_MetaEsc].nr = RC_META;
       break;
     case RC_CHDIR:
       s = *args ? *args : home;
@@ -904,6 +1135,7 @@ int key;
     case RC_LOGDIR:
       (void)ParseSaveStr(act, &screenlogdir);
       break;
+    case RC_SHELLTITLE:
     case RC_SHELLAKA:
       (void)ParseSaveStr(act, &nwin_default.aka);
       break;
@@ -961,7 +1193,7 @@ int key;
       (void)ParseSaveStr(act, &PowDetachString);
       break;
 #endif
-#ifdef UTMPOK
+#if defined(UTMPOK) && defined(LOGOUTOK)
     case RC_DEFLOGIN:
       (void)ParseOnOff(act, &nwin_default.lflag);
       break;
@@ -1014,7 +1246,7 @@ int key;
 	  Msg(0, "%s: releasing console %s", rc_name, HostName);
       else if (console_window)
 	  Msg(0, "%s: stealing console %s from window %d (%s)", rc_name, 
-	      HostName, console_window->w_number, console_window->w_aka);
+	      HostName, console_window->w_number, console_window->w_title);
       else
 	  Msg(0, "%s: grabbing console %s", rc_name, HostName);
       console_window = n ? fore : 0;
@@ -1022,14 +1254,11 @@ int key;
     case RC_ALLPARTIAL:
       if (ParseOnOff(act, &all_norefresh))
 	break;
-      if (all_norefresh)
-	Msg(0, "No refresh on window change!\n");
-      else
-	{
-	  if (fore)
-	    Activate(-1);
-	  Msg(0, "Window specific refresh\n");
-	}
+      if (!all_norefresh && fore)
+	Activate(-1);
+      if (msgok)
+        Msg(0, all_norefresh ? "No refresh on window change!\n" :
+			       "Window specific refresh\n");
       break;
     case RC_PARTIAL:
       (void)ParseSwitch(act, &n);
@@ -1082,7 +1311,7 @@ int key;
       break;
     case RC_NUMBER:
       if (*args == 0)
-        Msg(0, "This is window %d (%s).\n", fore->w_number, fore->w_aka);
+        Msg(0, "This is window %d (%s).\n", fore->w_number, fore->w_title);
       else
         {
 	  int old = fore->w_number;
@@ -1125,7 +1354,7 @@ int key;
 	  else
 #endif
 	    Msg(0, "Window %d (%s) is now being monitored for %d sec. silence.",
-	    	fore->w_number, fore->w_aka, fore->w_tstamp.seconds);
+	    	fore->w_number, fore->w_title, fore->w_tstamp.seconds);
 	}
       else
         {
@@ -1139,7 +1368,7 @@ int key;
 	  else
 #endif
 	    Msg(0, "Window %d (%s) is no longer being monitored for silence.", 
-		fore->w_number, fore->w_aka);
+		fore->w_number, fore->w_title);
 	}
       break;
 #ifdef COPY_PASTE
@@ -1188,6 +1417,12 @@ int key;
 	}
       break;
     case RC_SETENV:
+      if (!args[0] || !args[1])
+        {
+	  debug1("RC_SETENV arguments missing: %s\n", args[0] ? args[0] : "");
+          InputSetenv(args[0]);
+	}
+      else
 #ifndef USESETENV
 	{
 	  char *buf;
@@ -1380,6 +1615,21 @@ int key;
     }
 }
 
+void
+DoCommand(argv) 
+char **argv;
+{
+  struct action act;
+
+  if ((act.nr = FindCommnr(*argv)) == RC_ILLEGAL)  
+    {
+      Msg(0, "%s: unknown command '%s'", rc_name, *argv);
+      return;
+    }
+  act.args = argv + 1;
+  DoAction(&act, -1);
+}
+
 static char **
 SaveArgs(args)
 char **args;
@@ -1411,17 +1661,37 @@ char *buf, **args;
     {
       while (*p && (*p == ' ' || *p == '\t'))
 	++p;
-#ifdef PSEUDOS
-      if (argc == 0 && (*p == '!' || *p == '|'))
+      if (argc == 0)
 	{
-	  *ap++ = "exec";
-	  if (*p == '!')
-	    p++;
-	  while (*p == ' ')
-	    p++;
-	  argc++;
-	}
+	  /* 
+	   * Expand hardcoded shortcuts.
+	   * This should not be done here, cause multiple commands per
+	   * line or prefixed commands won't be recognized.
+	   * But as spaces between shortcut character and arguments
+	   * can be ommited this expansion affects tokenisation and
+	   * should be done here. Hmmm. jw.
+	   */
+	  switch (*p)
+	    {
+	    case '@':
+	      *ap++ = "at";
+	      while (*(++p) == ' ' || *p == '\t')
+	        ;
+	      argc++;
+	      break;
+#ifdef PSEUDOS
+	    case '!':
+	    case '|':
+	      *ap++ = "exec";
+	      if (*p == '!')
+		p++;
+	      while (*p == ' ' || *p == '\t')
+		p++;
+	      argc++;
+	      break;
 #endif
+	    }
+        }
       if (*p == '\0' || *p == '#')
 	{
 	  *p = '\0';
@@ -1453,11 +1723,12 @@ char *buf, **args;
 }
 
 int 
-ParseEscape(p)
+ParseEscape(u, p)
+struct user *u;
 char *p;
 {
-  if ((p = ParseChar(p, &Esc)) == NULL ||
-      (p = ParseChar(p, &MetaEsc)) == NULL || *p)
+  if ((p = ParseChar(p, u ? &u->u_Esc : &DefaultEsc)) == NULL ||
+      (p = ParseChar(p, u ? &u->u_MetaEsc : &DefaultMetaEsc)) == NULL || *p)
     return -1;
   return 0;
 }
@@ -1556,11 +1827,27 @@ char *s;
   struct win *p;
 
   for (p = windows; p; p = p->w_next)
-    if (!strncmp(p->w_aka, s, strlen(s)))
+    if (!strncmp(p->w_title, s, strlen(s)))
       return p;
   return NULL;
 }
 
+static int
+WindowByNumber(str)
+char *str;
+{
+  int i;
+  char *s;
+
+  for (i = 0, s = str; *s; s++)
+    {
+      if (*s < '0' || *s > '9')
+        break;
+      i = i * 10 + (*s - '0');
+    }
+  return *s ? -1 : i;
+}
+
 /* 
  * Get window number from Name or Number string.
  * Numbers are tried first, then names, a prefix match suffices.
@@ -1571,16 +1858,9 @@ WindowByNoN(str)
 char *str;
 {
   int i;
-  char *s;
   struct win *p;
   
-  for (i = 0, s = str; *s; s++)
-    {
-      if (*s < '0' || *s > '9')
-	break;
-      i = i * 10 + (*s - '0');
-    }
-  if (*s || i < 0 || i >= MAXWIN)
+  if ((i = WindowByNumber(str)) < 0 || i >= MAXWIN)
     {
       if ((p = WindowByName(str)))
 	return p->w_number;
@@ -1720,18 +2000,46 @@ int n;
     }
   if (p == d_fore)
     {
-      Msg(0, "This IS window %d (%s).", n, p->w_aka);
+      Msg(0, "This IS window %d (%s).", n, p->w_title);
       return;
     }
   if (p->w_display)
     {
-      Msg(0, "Window %d (%s) is on another display.", n, p->w_aka);
+      Msg(0, "Window %d (%s) is on another display (%s@%s).", n, p->w_title,
+          p->w_display->_d_user->u_name, p->w_display->_d_usertty);
       return;
     }
   SetForeWindow(p);
   Activate(fore->w_norefresh);  
 }  
 
+/* 
+ * returns 0, if the lock really has been released 
+ */
+int
+ReleaseAutoWritelock(dis, w)
+struct display *dis;
+struct win *w;
+{
+  /* release auto writelock when user has no other display here */
+  if (w->w_wlock == WLOCK_AUTO && w->w_wlockuser == d_user)
+    {
+      struct display *d;
+
+      for (d = displays; d; d = d->_d_next)
+	if (( d != display) && (d->_d_fore == w))
+	  break;
+      debug3("%s %s autolock on win %d\n", 
+	     d_user->u_name, d?"keeps":"releases", w->w_number);
+      if (!d)
+        {
+	  w->w_wlockuser = NULL;
+	  return 0;
+	}
+    }
+  return 1;
+}
+
 void
 SetForeWindow(wi)
 struct win *wi;
@@ -1746,19 +2054,7 @@ struct win *wi;
       fore = d_fore;
       if (fore)
 	{
-	  /* release auto writelock when user has no other display here */
-	  if (fore->w_wlock == WLOCK_AUTO && fore->w_wlockuser == d_user)
-	    {
-	      struct display *d;
-
-	      for (d = displays; d; d = d->_d_next)
-	        if (( d != display) && (d->_d_fore == fore))
-		  break;
-	      debug3("%s %s autolock on win %d\n", 
-	             d_user->u_name, d?"keeps":"releases", fore->w_number);
-	      if (!d)
-	        fore->w_wlockuser = NULL;
-	    }
+	  ReleaseAutoWritelock(display, fore);
 	  /* deactivate old window. */
 	  if (fore->w_tstamp.seconds)
 	    fore->w_tstamp.lastio = Now;
@@ -1963,7 +2259,7 @@ ShowWindows()
       if ((p = *pp) == 0)
 	continue;
 
-      cmd = p->w_aka;
+      cmd = p->w_title;
       if (s - buf + strlen(cmd) > sizeof(buf) - 6)
 	break;
       if (s > buf)
@@ -1995,6 +2291,8 @@ ShowWindows()
 	  strcpy(s, "(L)");
 	  s += 3;
 	}
+      if (p->w_ptyfd < 0)
+        *s++ = 'Z';
       *s++ = ' ';
       strcpy(s, cmd);
       s += strlen(s);
@@ -2100,7 +2398,7 @@ int len;
 static void
 InputAKA()
 {
-  Input("Set window's a.k.a. to: ", 20, AKAfin, INP_COOKED);
+  Input("Set window's title to: ", 20, AKAfin, INP_COOKED);
 }
 
 static void
@@ -2138,6 +2436,54 @@ InputSelect()
   Input("Switch to window: ", 20, SelectFin, INP_COOKED);
 }
 
+static char setenv_var[31];
+
+
+static void
+SetenvFin1(buf, len)
+char *buf;
+int len;
+{
+  if (!len || !display)
+    return;
+  InputSetenv(buf);
+}
+  
+static void
+SetenvFin2(buf, len)
+char *buf;
+int len;
+{
+  struct action act;
+  char *args[3];
+
+  if (!len || !display)
+    return;
+  act.nr = RC_SETENV;
+  args[0] = setenv_var;
+  args[1] = buf;
+  args[2] = NULL;
+  act.args = args;
+  debug2("SetenvFin2: setenv '%s' '%s'\n", setenv_var, buf);
+  DoAction(&act, -1);
+}
+
+static void
+InputSetenv(arg)
+char *arg;
+{
+  static char setenv_buf[80];	/* need to be static here, cannot be freed */
+
+  if (arg)
+    {
+      strncpy(setenv_var, arg, 30);
+      sprintf(setenv_buf, "Enter value for %s: ", arg);
+      Input(setenv_buf, 30, SetenvFin2, INP_COOKED);
+    }
+  else
+    Input("Setenv: Enter variable name: ", 30, SetenvFin1, INP_COOKED);
+}
+
 void
 DoScreen(fn, av)
 char *fn, **av;
@@ -2197,6 +2543,7 @@ char *fn, **av;
 	  else 
 	    --av;
 	  break;
+#ifdef LOGOUTOK
 	case 'l':
 	  switch (av[0][2])
 	    {
@@ -2213,6 +2560,7 @@ char *fn, **av;
 	      break;
 	    }
 	  break;
+#endif
 	case 'a':
 	  nwin.aflag = 1;
 	  break;
@@ -2337,14 +2685,14 @@ int len;
     }
   if (pp->buf)
     free(pp->buf);
-  if ((pp->buf = (char *)malloc(d_copylen)) == NULL)
+  if ((pp->buf = (char *)malloc(d_user->u_copylen)) == NULL)
     {
       Msg(0, strnomem);
       return;
     }
-  bcopy(d_copybuffer, pp->buf, d_copylen);
-  pp->len = d_copylen;
-  Msg(0, "Copied %d characters into register %c", d_copylen, *buf);
+  bcopy(d_user->u_copybuffer, pp->buf, d_user->u_copylen);
+  pp->len = d_user->u_copylen;
+  Msg(0, "Copied %d characters into register %c", d_user->u_copylen, *buf);
 }
 
 static void
@@ -2361,7 +2709,10 @@ int len;
     }
   if (pp->buf)
     {
-      fore->w_pastebuffer  = pp->buf;
+      if (fore->w_pastebuf)
+        free(fore->w_pastebuf);
+      fore->w_pastebuf = 0;
+      fore->w_pasteptr = pp->buf;
       fore->w_pastelen = pp->len;
       return;
     }
@@ -2442,17 +2793,17 @@ int len;
   if (CheckPassword)
     {
 #ifdef COPY_PASTE
-      if (d_copybuffer)
-	free(d_copybuffer);
-      d_copylen = strlen(Password);
-      if ((d_copybuffer = (char *) malloc(d_copylen + 1)) == NULL)
+      if (d_user->u_copybuffer)
+	UserFreeCopyBuffer(d_user);
+      d_user->u_copylen = strlen(Password);
+      if ((d_user->u_copybuffer = (char *) malloc(d_user->u_copylen + 1)) == NULL)
 	{
 	  Msg(0, strnomem);
-          d_copylen = 0;
+          d_user->u_copylen = 0;
 	}
       else
 	{
-	  strcpy(d_copybuffer, Password);
+	  strcpy(d_user->u_copybuffer, Password);
 	  Msg(0, "[ Password moved into copybuffer ]");
 	}
 #else				/* COPY_PASTE */
