@@ -21,7 +21,7 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: pty.c,v 1.1.1.1 1993/06/16 23:51:15 jnweiger Exp $ FAU")
+RCS_ID("$Id: pty.c,v 1.3 1993/08/04 00:42:48 mlschroe Exp $ FAU")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -35,6 +35,10 @@ RCS_ID("$Id: pty.c,v 1.1.1.1 1993/06/16 23:51:15 jnweiger Exp $ FAU")
 #include <sys/ioctl.h>
 #endif
 
+#if defined(sun) && defined(LOCKPTY) && !defined(TIOCEXCL)
+#include <sys/ttold.h>
+#endif
+
 #ifdef ISC
 # include <sys/tty.h>
 # include <sys/sioctl.h>
@@ -100,7 +104,65 @@ char **ttyn;
 }
 
 #else
-# ifdef SVR4
+# ifdef MIPS
+#  ifdef __sgi /* __sgi -> IRIX 4.0 */
+
+int
+OpenPTY(ttyn)
+char **ttyn;
+{
+  int f;
+  char *name; 
+  sig_t (*sigcld)__P(SIGPROTOARG);
+
+  /*
+   * SIGCHLD set to SIG_DFL for _getpty() because it may fork() and
+   * exec() /usr/adm/mkpts
+   */
+  sigcld = signal(SIGCHLD, SIG_DFL);
+  name = _getpty(&f, O_RDWR | O_NDELAY, 0600, 0);
+  signal(SIGCHLD, sigcld);
+
+  if (name == NULL)
+    return -1;
+#ifdef LOCKPTY
+  (void) ioctl(f, TIOCEXCL, (char *) 0);
+#endif
+  *ttyn = name;
+  return f;
+}
+
+#  else /* __sgi */
+
+int
+OpenPTY(ttyn)
+char **ttyn;
+{
+  register int f;
+  register int my_minor;
+  struct stat buf;
+   
+  strcpy(PtyName, "/dev/ptc");
+  f = open(PtyName, O_RDWR | O_NDELAY);
+  if (f < 0)
+    return -1;
+  if (fstat(f, &buf) < 0)
+    {
+      close(f);
+      return -1;
+    }
+  my_minor = minor(buf.st_rdev);
+  sprintf(TtyName, "/dev/ttyq%d", my_minor);
+#ifdef LOCKPTY
+  (void) ioctl(f, TIOCEXCL, (char *) 0);
+#endif
+  *ttyn = TtyName;
+  return f;
+}
+
+#  endif /* __sgi */
+# else /* MIPS */
+#  ifdef SVR4
 
 int
 OpenPTY(ttyn)
@@ -143,67 +205,7 @@ char **ttyn;
   return f;
 }
 
-# else /* not SVR4 */
-
-#  ifdef MIPS
-#   ifdef __sgi /* __sgi -> IRIX 4.0 */
-
-int
-OpenPTY(ttyn)
-char **ttyn;
-{
-  int f;
-  char *name; 
-  sig_t (*sigcld)__P(SIGPROTOARG);
-
-  /*
-   * SIGCHLD set to SIG_DFL for _getpty() because it may fork() and
-   * exec() /usr/adm/mkpts
-   */
-  sigcld = signal(SIGCHLD, SIG_DFL);
-  name = _getpty(&f, O_RDWR | O_NDELAY, 0600, 0);
-  signal(SIGCHLD, sigcld);
-
-  if (name == NULL)
-    return -1;
-#ifdef LOCKPTY
-  (void) ioctl(f, TIOCEXCL, (char *) 0);
-#endif
-  *ttyn = name;
-  return f;
-}
-
-#   else /* __sgi */
-
-int
-OpenPTY(ttyn)
-char **ttyn;
-{
-  register int f;
-  register int my_minor;
-  struct stat buf;
-   
-  strcpy(PtyName, "/dev/ptc");
-  f = open(PtyName, O_RDWR | O_NDELAY);
-  if (f < 0)
-    return -1;
-  if (fstat(f, &buf) < 0)
-    {
-      close(f);
-      return -1;
-    }
-  my_minor = minor(buf.st_rdev);
-  sprintf(TtyName, "/dev/ttyq%d", my_minor);
-#ifdef LOCKPTY
-  (void) ioctl(f, TIOCEXCL, (char *) 0);
-#endif
-  *ttyn = TtyName;
-  return f;
-}
-
-#   endif /* __sgi */
-#  else /* MIPS */
-
+#  else /* not SVR4 */
 #   ifdef _AIX /* RS6000 */
 
 int
@@ -262,7 +264,7 @@ char **ttyn;
 	      close(f);
 	      continue;
 	    }
-#if defined(sun) && defined(TIOCGPGRP)
+#if defined(sun) && defined(TIOCGPGRP) && !defined(SUNOS3)
 	  /* Hack to ensure that the slave side of the pty is
 	   * unused. May not work in anything other than SunOS4.1
 	   */
@@ -270,7 +272,7 @@ char **ttyn;
 	      int pgrp;
 
 	      /* tcgetpgrp does not work (uses TIOCGETPGRP)! */
-	      if (ioctl(f, TIOCGPGRP, &pgrp) != -1 || errno != EIO)
+	      if (ioctl(f, TIOCGPGRP, (char *)&pgrp) != -1 || errno != EIO)
 		{
 		  close(f);
 		  continue;
@@ -288,6 +290,6 @@ char **ttyn;
 }
 
 #   endif /* _AIX, RS6000 */
-#  endif /* MIPS */
-# endif /* SVR4 */
+#  endif /* SVR4 */
+# endif /* MIPS */
 #endif /* sequent || SEQUENT */
