@@ -21,25 +21,20 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: screen.c,v 1.9 1993/07/09 12:07:49 jnweiger Exp $ FAU")
+RCS_ID("$Id: screen.c,v 1.15 1993/08/05 14:24:06 mlschroe Exp $ FAU")
 
 
 #include <ctype.h>
 #ifdef __sgi
 # include <stdio.h> /* needed before pwd.h to avoid ansi compiler whining */
 #endif /* __sgi */
+
 #include <pwd.h>
 #include <fcntl.h>
 #ifdef sgi
 # include <sys/sysmacros.h>
 #endif /* sgi */
-#if !defined(sun) && !defined(B43) && !defined(ISC) && !defined(pyr) && !defined(_CX_UX)
-# include <time.h>
-#endif
-#include <sys/time.h>
-#if defined(M_XENIX) || defined(M_UNIX)
-#include <sys/select.h> /* for timeval */
-#endif
+
 #include <sys/types.h>
 #ifdef ISC
 # include <sys/bsdtypes.h>
@@ -82,35 +77,6 @@ RCS_ID("$Id: screen.c,v 1.9 1993/07/09 12:07:49 jnweiger Exp $ FAU")
 
 #include "patchlevel.h"
 
-#if defined(xelos) || defined(m68k) || defined(M_XENIX)
- struct passwd *getpwuid __P((uid_t));
- struct passwd *getpwnam __P((char *));
-#endif
-
-#ifdef USEVARARGS
-# if defined(__STDC__)
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
-#endif
-
-#if defined(_SEQUENT_) 
-/* for the FD.. stuff */
-# include <sys/select.h>
-#endif /* _SEQUENT_ */
-
-#ifndef FD_SET
-typedef struct fd_set
-{
-  int fd_bits[1];
-}      fd_set;
-# define FD_ZERO(fd) ((fd)->fd_bits[0] = 0)
-# define FD_SET(b, fd) ((fd)->fd_bits[0] |= 1 << (b))
-# define FD_ISSET(b, fd) ((fd)->fd_bits[0] & 1 << (b))
-# define FD_SETSIZE 32
-#endif
-
 
 #ifdef DEBUG
 FILE *dfp;
@@ -121,6 +87,8 @@ extern char *blank, *null, Term[], screenterm[], **environ, Termcap[];
 int force_vt = 1, assume_LP = 0;
 int VBellWait, MsgWait, MsgMinWait, SilenceWait;
 
+extern struct plop plop_tab[];
+extern struct user *users;
 extern struct display *displays, *display; 
 extern struct layer BlankLayer;
 
@@ -207,6 +175,7 @@ int MasterPid;
 int real_uid, real_gid, eff_uid, eff_gid;
 int default_startup;
 int slowpaste;
+int ZombieKey;
 
 #ifdef NETHACK
 int nethackflag = 0;
@@ -218,22 +187,12 @@ struct win *windows = NULL;
 struct win *console_window;
 
 
+
 /*
  * Do this last
  */
 #include "extern.h"
 
-/*
- * XXX: Missing system header files.
- */
-#ifdef USEVARARGS
-# ifndef VPRNT_DECLARED
-int vsprintf __P((char *, char *, va_list));
-# endif /* VPRNT_DECLARED */
-#endif
-#ifndef SELECT_DECLARED
-int select __P((int, fd_set *, fd_set *, fd_set *, const struct timeval *));
-#endif
 
 #ifdef NETHACK
 char strnomem[] = "Who was that Maude person anyway?";
@@ -283,6 +242,8 @@ fd_set *rp, *wp;
     }
   for (p = windows; p; p = p->w_next)
     {
+      if (p->w_ptyfd < 0)
+        continue;
 #ifdef COPY_PASTE
       if (p->w_pastelen)
         {
@@ -385,11 +346,16 @@ char **av;
    */
   closeallfiles(0);
 #ifdef DEBUG
-  (void) mkdir("/tmp/debug", 0777);
-  if ((dfp = fopen("/tmp/debug/screen.front", "w")) == NULL)
-    dfp = stderr;
-  else
-    (void) chmod("/tmp/debug/screen.front", 0666);
+    {
+      char buf[255];
+
+      sprintf(buf, "%s/screen.%d", DEBUGDIR, getpid());
+      (void) mkdir(DEBUGDIR, 0777);
+      if ((dfp = fopen(buf, "w")) == NULL)
+	dfp = stderr;
+      else
+	(void) chmod(buf, 0666);
+    }
 #endif
   sprintf(version, "%d.%.2d.%.2d%s (%s) %s", REV, VERS,
 	  PATCHLEVEL, STATE, ORIGIN, DATE);
@@ -505,7 +471,7 @@ char **av;
 		    exit_with_usage(myname);
 		  ap = *++av;
 		}
-	      if (ParseEscape(ap))
+	      if (ParseEscape(NULL, ap))
 		Panic(0, "Two characters are required with -e option.");
 	      break;
 	    case 'f':
@@ -1080,12 +1046,17 @@ char **av;
     *av0 = 'S';
 
 #ifdef DEBUG
-  if (dfp != stderr)
-    fclose(dfp);
-  if ((dfp = fopen("/tmp/debug/screen.back", "w")) == NULL)
-    dfp = stderr;
-  else
-    (void) chmod("/tmp/debug/screen.back", 0666);
+  {
+    char buf[256];
+
+    if (dfp && dfp != stderr)
+      fclose(dfp);
+    sprintf(buf, "%s/SCREEN.%d", DEBUGDIR, getpid());
+    if ((dfp = fopen(buf, "w")) == NULL)
+      dfp = stderr;
+    else
+      (void) chmod(buf, 0666);
+  }
 #endif
   if (!detached)
     n = dup(0);
@@ -1125,10 +1096,12 @@ char **av;
   SockName = socknamebuf;
   ServerSocket = MakeServerSocket();
 #ifdef ETCSCREENRC
-  if ((ap = getenv("SYSSCREENRC")) == NULL)
-    StartRc(ETCSCREENRC);
-  else
+# ifdef ALLOW_SYSSCREENRC
+  if ((ap = getenv("SYSSCREENRC")))
     StartRc(ap);
+  else
+# endif
+    StartRc(ETCSCREENRC);
 #endif
   StartRc(RcFileName);
 # ifdef UTMPOK
@@ -1181,10 +1154,12 @@ char **av;
   else
     brktty(-1);		/* just try */
 #ifdef ETCSCREENRC
-  if ((ap = getenv("SYSSCREENRC")) == NULL)
-    FinishRc(ETCSCREENRC);
-  else
+# ifdef ALLOW_SYSSCREENRC
+  if ((ap = getenv("SYSSCREENRC")))
     FinishRc(ap);
+  else
+# endif
+    FinishRc(ETCSCREENRC);
 #endif
   FinishRc(RcFileName);
 
@@ -1344,6 +1319,9 @@ char **av;
 	  for (p = windows; p; p = p->w_next)
 	    {
 	      int pastefd = -1;
+
+	      if (p->w_ptyfd < 0)
+	        continue;
 #ifdef COPY_PASTE
 	      if (p->w_pastelen)
 		{
@@ -1358,18 +1336,25 @@ char **av;
 		  if (FD_ISSET(pastefd, &w))
 		    {
 		      debug1("writing pastebuffer (%d)\n", p->w_pastelen);
-		      len = write(pastefd, p->w_pastebuffer, 
-		                  (slowpaste > 0) ? 1 :
+		      len = write(pastefd, p->w_pasteptr, 
+				  (slowpaste > 0) ? 1 :
 				  (p->w_pastelen > IOSIZE ? 
 				   IOSIZE : p->w_pastelen));
 		      if (len < 0)	/* Problems... window is dead */
 			p->w_pastelen = 0;
 		      if (len > 0)
 			{
-			  p->w_pastebuffer += len;
+			  p->w_pasteptr += len;
 			  p->w_pastelen -= len;
 			}
 		      debug1("%d bytes pasted\n", len);
+		      if (p->w_pastelen == 0)
+			{
+			  if (p->w_pastebuf)
+			    free(p->w_pastebuf);
+			  p->w_pasteptr = 0;
+			  pastefd = -1;
+			}
 		      if (slowpaste > 0)
 			{
 			  struct timeval t;
@@ -1381,8 +1366,6 @@ char **av;
 			}
 		      if (--nsel == 0)
 		        break;
-		      if (p->w_pastelen == 0)
-			pastefd = -1;
 		    }
 		}
 #endif
@@ -1494,7 +1477,7 @@ char **av;
 		{
 		  if (maxlen == 1)
 		    maxlen = 2;	/* Allow one char for command keys */
-		  buf[0] = Esc;
+		  buf[0] = d_user->u_Esc;
 		  buflen = read(d_userfd, buf + 1, maxlen - 1) + 1;
 		  d_ESCseen = 0;
 		}
@@ -1534,7 +1517,7 @@ char **av;
 	  display = p->w_display;
 	  if (p->w_outlen)
 	    WriteString(p, p->w_outbuf, p->w_outlen);
-	  else 
+	  else if (p->w_ptyfd >= 0)
 	    {
 #ifdef PSEUDOS
 	      /* gather pseudowin output */
@@ -1600,7 +1583,15 @@ char **av;
 			continue;
 #endif
 		      debug2("Window %d: read error (errno %d) - killing window\n", p->w_number, len ? errno : 0);
-		      KillWindow(p);
+		      if (ZombieKey)
+			{
+			  debug3("window %d (%s) going into zombie state fd %d",
+				 p->w_number, p->w_title, p->w_ptyfd);
+			  p->w_ptyfd = -1;
+			  debug("changed to -1\n");
+			}
+		      else
+		        KillWindow(p);
 		      nsel = 0;	/* KillWindow may change window order */
 		      break;	/* so we just break */
 		    }
@@ -1872,7 +1863,17 @@ DoWait()
 		}
 	      else
 #endif
-		KillWindow(p);
+		{
+		  if (ZombieKey)
+		    {
+		      debug3("window %d (%s) going into zombie state fd %d",
+		             p->w_number, p->w_title, p->w_ptyfd);
+		      p->w_ptyfd = -1;
+		      debug("changed to -1\n");
+		    }
+		  else
+		    KillWindow(p);
+		}
 	      break;
 	    }
 #ifdef PSEUDOS
@@ -2059,10 +2060,14 @@ int mode;
 #endif
   if (d_fore)
     {
+      ReleaseAutoWritelock(display, d_fore);
+      if (d_fore->w_tstamp.seconds)
+        d_fore->w_tstamp.lastio = Now;
       d_fore->w_active = 0;
       d_fore->w_display = 0;
       d_lay = &BlankLayer;
       d_layfn = d_lay->l_layfn;
+      d_user->u_detachwin = d_fore->w_number;
     }
   while (d_lay != &BlankLayer)
     ExitOverlayPage();
@@ -2128,6 +2133,7 @@ MakeNewEnv()
 
   for (op = environ; *op; ++op)
     {
+debug1("MakeNewEnv: %s\n", *op);
       if (!IsSymbol(*op, "TERM") && !IsSymbol(*op, "TERMCAP")
 	  && !IsSymbol(*op, "STY") && !IsSymbol(*op, "WINDOW")
 	  && !IsSymbol(*op, "SCREENCAP") && !IsSymbol(*op, "SHELL")
