@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  ****************************************************************
- * $Id: screen.h,v 1.5 1993/07/21 15:43:23 mlschroe Exp $ FAU
+ * $Id: screen.h,v 1.8 1993/08/04 00:42:57 mlschroe Exp $ FAU
  */
 
 #include "os.h"
@@ -34,6 +34,8 @@
 # define const
 #endif
 
+#include "osdef.h"
+
 #include "ansi.h"
 #include "acl.h"
 #include "comm.h"
@@ -42,10 +44,11 @@
 
 
 #ifdef DEBUG
-#	define debug(x) {fprintf(dfp,x);fflush(dfp);}
-#	define debug1(x,a) {fprintf(dfp,x,a);fflush(dfp);}
-#	define debug2(x,a,b) {fprintf(dfp,x,a,b);fflush(dfp);}
-#	define debug3(x,a,b,c) {fprintf(dfp,x,a,b,c);fflush(dfp);}
+#	define DEBUGDIR "/tmp/debug"
+#	define debug(x) {if(dfp){fprintf(dfp,x);fflush(dfp);}}
+#	define debug1(x,a) {if(dfp){fprintf(dfp,x,a);fflush(dfp);}}
+#	define debug2(x,a,b) {if(dfp){fprintf(dfp,x,a,b);fflush(dfp);}}
+#	define debug3(x,a,b,c) {if(dfp){fprintf(dfp,x,a,b,c);fflush(dfp);}}
 	extern FILE *dfp;
 #else
 #	define debug(x) {}
@@ -241,3 +244,13 @@ struct acl
 };
 #endif
 
+/* register list */
+#define MAX_PLOP_DEFS 256
+struct plop
+{
+  char *buf;
+  int len;
+};
+
+#define PRSTART '['
+#define PREND ']'
