@@ -21,7 +21,7 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: socket.c,v 1.6 1993/07/21 15:43:26 mlschroe Exp $ FAU")
+RCS_ID("$Id: socket.c,v 1.11 1993/08/04 00:43:00 mlschroe Exp $ FAU")
 
 #include "config.h"
 #include <sys/types.h>
@@ -69,7 +69,7 @@ extern int ServerSocket, real_uid, real_gid, eff_uid, eff_gid;
 extern int dflag, iflag, rflag, lsflag, quietflag, wipeflag, xflag;
 extern char *attach_tty, *LoginName, HostName[];
 extern struct display *display, *displays;
-extern struct win *fore, *console_window, *windows;
+extern struct win *fore, *wtab[], *console_window, *windows;
 extern struct NewWindow nwin_undef;
 #ifdef NETHACK
 extern nethackflag;
@@ -172,6 +172,7 @@ int *fdp;
        * there may be a file ".termcap" here. 
        * Ignore it just like "." and "..". 
        */
+      debug1("- %s\n", Name);
       if (Name[0] == '.')
 	continue;
       if (SockName && l)
@@ -207,7 +208,10 @@ int *fdp;
        */
       strcpy(SockNamePtr, Name);
       if (stat(SockPath, &st))
-	continue;
+        {
+          debug1("could not stat! (%d)\n", errno);
+	  continue;
+        }
 #ifndef SOCK_NOT_IN_FS
 # ifdef NAMEDPIPE
       if (!S_ISFIFO(st.st_mode))
@@ -226,8 +230,12 @@ int *fdp;
 # endif /* NAMEDPIPE */
 #endif
       if (st.st_uid != real_uid)
-	continue;
+        {
+          debug2("uid mismatch (%d - %d) - ignored\n", st.st_uid, real_uid);
+	  continue;
+        }
       s = st.st_mode & 0777;
+      debug2("FindSocket: %s has mode %04o...\n", Name, s);
 #ifdef MULTIUSER
       if (multi && ((s & 0677) == 0600))
 	continue;
@@ -238,7 +246,6 @@ int *fdp;
 #else
       foundsock[foundsockcount].mode = s;
 #endif
-      debug2("FindSocket: %s has mode %04o...\n", Name, s);
       {
 	/* 
 	 * marc parses the socketname again
@@ -584,7 +591,7 @@ MakeServerSocket()
 	Panic(0, "It is not detached.");
       /* NOTREACHED */
     }
-#if defined(m88k) || defined(m68k)
+#if defined(m88k) || defined(sysV68)
   close(s);	/* we get bind: Invalid argument if this is not done */
   if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
     Panic(errno, "reopen socket");
@@ -595,7 +602,7 @@ MakeServerSocket()
 #ifdef SOCK_NOT_IN_FS
     {
       int f;
-      if (f = secopen(SockPath, O_RDWR | O_CREAT, SOCKMODE) < 0)
+      if ((f = secopen(SockPath, O_RDWR | O_CREAT, SOCKMODE)) < 0)
         Panic(errno, "shadow socket open");
       close(f);
     }
@@ -1055,10 +1062,12 @@ ReceiveMsg()
       extra_incap = extra_outcap = 0;
       debug2("Message says size (%dx%d)\n", m.m.attach.columns, m.m.attach.lines);
 #ifdef ETCSCREENRC
-      if ((p = getenv("SYSSCREENRC")) == NULL)
-	StartRc(ETCSCREENRC);
-      else
+# ifdef ALLOW_SYSSCREENRC
+      if ((p = getenv("SYSSCREENRC")))
 	StartRc(p);
+      else
+# endif
+	StartRc(ETCSCREENRC);
 #endif
       StartRc(RcFileName);
       if (InitTermcap(m.m.attach.columns, m.m.attach.lines))
@@ -1084,12 +1093,36 @@ ReceiveMsg()
 #endif
       SetMode(&d_OldMode, &d_NewMode);
       SetTTY(d_userfd, &d_NewMode);
-      if (fore && fore->w_display == 0)
+
+      d_fore = NULL;
+      if (d_user->u_detachwin >= 0) 
+        fore = wtab[d_user->u_detachwin];
+      if (!fore || fore->w_display
+#ifdef MULTIUSER
+				   || AclCheckPermWin(d_user, ACL_WRITE, fore)
+#endif
+				   					      )
+        {
+	  /* try to get another window */
+#ifdef MULTIUSER
+	  for (wi = windows; wi; wi = wi->w_next)
+	    if (!wi->w_display && !AclCheckPermWin(d_user, ACL_WRITE, fore))
+	      break;
+	  if (!wi)
+	    for (wi = windows; wi; wi = wi->w_next)
+	      if (!wi->w_display && !AclCheckPermWin(d_user, ACL_READ, fore))
+	        break;
+	  if (!wi)
+#endif
+	    for (wi = windows; wi; wi = wi->w_next)
+	      if (!wi->w_display)
+	        break;
+	  fore = wi;
+	}
+      if (fore)
         SetForeWindow(fore);
-      else
-        d_fore = 0;
       Activate(0);
-      if (d_fore == 0)
+      if (!d_fore)
 	ShowWindows();
       if (displays->_d_next == 0 && console_window)
 	{
