@@ -21,7 +21,7 @@
  */
 
 #include "rcs.h"
-RCS_ID("$Id: window.c,v 1.9 1993/07/21 15:43:41 mlschroe Exp $ FAU")
+RCS_ID("$Id: window.c,v 1.14 1993/08/05 14:24:15 mlschroe Exp $ FAU")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -42,6 +42,7 @@ RCS_ID("$Id: window.c,v 1.9 1993/07/21 15:43:41 mlschroe Exp $ FAU")
 extern struct display *displays, *display;
 extern struct win *windows, *fore, *wtab[], *console_window;
 extern char *ShellArgs[];
+extern char *ShellProg;
 extern char screenterm[];
 extern char HostName[];
 extern int default_monitor, TtyMode;
@@ -149,7 +150,7 @@ struct NewWindow *newwin;
       Msg(0, strnomem);
       return -1;
     }
-  bzero((char *) p, (int) sizeof(struct win));
+  bzero((char *) p, (int) sizeof(struct win)); /* looks like a calloc above */
 #ifdef MULTIUSER
   if (NewWindowAcl(p))
     {
@@ -179,11 +180,11 @@ struct NewWindow *newwin;
     {
       p->w_autoaka = 0;
       *nwin.aka++ = 0;
-      p->w_aka = nwin.aka;
+      p->w_title = nwin.aka;
       p->w_akachange = nwin.aka + strlen(nwin.aka);
     }
   else
-    p->w_aka = p->w_akachange = p->w_akabuf;
+    p->w_title = p->w_akachange = p->w_akabuf;
   if ((p->w_monitor = nwin.monitor) == -1)
     p->w_monitor = default_monitor;
   p->w_norefresh = 0;
@@ -305,7 +306,7 @@ char **namep;
       {
 	int flag = 1;
 
-	if (ioctl(f, TIOCPKT, &flag))
+	if (ioctl(f, TIOCPKT, (char *)&flag))
 	  {
 	    Msg(errno, "TIOCPKT ioctl");
 	    close(f);
@@ -356,6 +357,12 @@ struct win *win;
   struct pseudowin *pwin = win->w_pwin;
 #endif
 
+  proc = *args;
+  if (proc == 0)
+    {
+      args = ShellArgs;
+      proc = *args;
+    }
   switch (pid = fork())
     {
     case -1:
@@ -389,15 +396,20 @@ struct win *win;
       else
 	brktty(-1);
 #ifdef DEBUG
-      if (dfp != stderr)
+      if (dfp && dfp != stderr)
 	fclose(dfp);
 #endif
       closeallfiles(win->w_ptyfd);
 #ifdef DEBUG
-      if ((dfp = fopen("/tmp/debug/screen.child", "a")) == 0)
-	dfp = stderr;
-      else
-	(void) chmod("/tmp/debug/screen.child", 0666);
+	{
+	  char buf[256];
+	  
+	  sprintf(buf, "%s/screen.child", DEBUGDIR);
+	  if ((dfp = fopen(buf, "a")) == 0)
+	    dfp = stderr;
+	  else
+	    (void) chmod(buf, 0666);
+	}
       debug1("=== ForkWindow: pid %d\n", getpid());
 #endif
       /* Close the three /dev/null descriptors */
@@ -458,7 +470,7 @@ struct win *win;
       if (newfd >= 0)
 	{
 	  struct mode fakemode, *modep;
-#ifdef SVR4
+#if defined(SVR4) && !defined(sgi)
 	  if (ioctl(newfd, I_PUSH, "ptem"))
 	    {
 	      SendErrorMsg("Cannot I_PUSH ptem %s %s", ttyn, sys_errlist[errno]);
@@ -510,7 +522,7 @@ struct win *win;
 #ifdef TIOCSWINSZ
 	  glwz.ws_col = w;
 	  glwz.ws_row = h;
-	  (void) ioctl(newfd, TIOCSWINSZ, &glwz);
+	  (void) ioctl(newfd, TIOCSWINSZ, (char *)&glwz);
 #endif
 	}
 #ifndef TIOCSWINSZ
@@ -523,13 +535,11 @@ struct win *win;
 	NewEnv[2] = MakeTermcap(1);
       else
 	NewEnv[2] = Termcap;
-      proc = *args;
-      if (*proc == '-')
-	proc++;
       strcpy(shellbuf, "SHELL=");
-      strncpy(shellbuf + 6, proc, MAXPATHLEN);
+      strncpy(shellbuf + 6, ShellProg, MAXPATHLEN);
       shellbuf[MAXPATHLEN + 6] = 0;
       NewEnv[4] = shellbuf;
+      debug1("ForkWindow: NewEnv[4] = '%s'\n", shellbuf);
       if (term && *term && strcmp(screenterm, term) &&
 	  (strlen(term) < 20))
 	{
@@ -554,6 +564,8 @@ struct win *win;
       sprintf(ebuf, "WINDOW=%d", win->w_number);
       NewEnv[3] = ebuf;
 
+      if (*proc == '-')
+	proc++;
       debug1("calling execvpe %s\n", proc);
       execvpe(proc, args, NewEnv);
       debug1("exec error: %d\n", errno);
@@ -631,6 +643,11 @@ char **av;
       Msg(0, "Filter running: %s", w->w_pwin ? w->w_pwin->p_cmd : "(none)");
       return -1;
     }
+  if (w->w_ptyfd < 0)
+    {
+      Msg(0, "You feel dead inside.");
+      return -1;
+    }
   if (!(pwin = (struct pseudowin *)malloc(sizeof(struct pseudowin))))
     {
       Msg(0, strnomem);
@@ -720,7 +737,7 @@ char **av;
   {
     int flag = 0;
 
-    if (ioctl(pwin->p_ptyfd, TIOCPKT, &flag))
+    if (ioctl(pwin->p_ptyfd, TIOCPKT, (char *)&flag))
       {
 	Msg(errno, "TIOCPKT ioctl");
 	FreePseudowin(w);
@@ -800,18 +817,26 @@ char **av;
       Msg(errno, "fork");
       return -1;
     case 0:
-      setuid(real_uid);
-      setgid(real_gid);
-      d_usertty[0] = 0;
+      d_usertty[0] = 0;		/* for SendErrorMsg */
+      if (setuid(real_uid) || setgid(real_gid))
+	{
+	  SendErrorMsg("Setuid/gid: %s", sys_errlist[errno]);
+	  eexit(1);
+	}
       closeallfiles(sf);
       close(1);
       dup(sf);
       close(sf);
 #ifdef DEBUG
-      if ((dfp = fopen("/tmp/debug/screen.child", "a")) == 0)
-	dfp = stderr;
-      else
-	(void) chmod("/tmp/debug/screen.child", 0666);
+	{
+	  char buf[256];
+
+	  sprintf(buf, "%s/screen.child", DEBUGDIR);
+	  if ((dfp = fopen(buf, "a")) == 0)
+	    dfp = stderr;
+	  else
+	    (void) chmod(buf, 0666);
+	}
       debug1("=== Clone: pid %d\n", getpid());
 #endif
       for (avp = av; *avp; avp++)
